#include "../C-Engine/CEngine/Object.hpp"
#include "../C-Engine/CEngine/Engine.hpp"
#include "../C-Engine/CEngine/Script.hpp"

#include "../C-Engine/CPhysics/Transforms.hpp"
#include "../C-Engine/CPhysics/Rigibodys.hpp"

#include "../C-Engine/CRendering/Render.hpp"
#include "../C-Engine/CRendering/Animation.hpp"
#include "../C-Engine/CRendering/Sprite.hpp"
#include "../C-Engine/CRendering/TilesMapping.hpp"

extern "C"
{
#include <fxlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../C-Engine/CExtraLib/input.h"
#include "../C-Engine/CExtraLib/MonochromeLib.h"
#include "../C-Engine/CExtraLib/FonctionC.h"
}

Engine::Engine()
{
    this->execute = true;
    this->run = true;
    this->fps = false;
    this->fpswish = 4;
    this->ListeObject = NULL;
    this->EScript = NULL;
    this->restart = false;
    this->time = 0;
    this->nbobject = 0;
    this->gamemap = NULL;
    this->type = 0;

    screen.min.x = 0;
    screen.max.x = 128;
    screen.min.y = 0;
    screen.max.y = 64;

}

int Engine::Game()
{

    srand(RTC_getTicks());

    if(EScript != NULL)EScript->Start();

    for(int i = 0; i< nbobject; i++)
        {
            if(ListeObject[i]->GetScript() != NULL && ListeObject[i]->GetEnable() == true)ListeObject[i]->GetScript()->Start();
        }

    while(execute)
        {
            ML_clear_vram();
            input_update();

            if(EScript != NULL)EScript->Update();

            if(WaitOneSecond())
            {
                if(EScript != NULL)EScript->UpdateEverySecond();

                for(int i = 0; i < nbobject; i++)
                {
                    if(ListeObject[i]->GetEnable() == true && i < nbobject)
                    {
                            if(ListeObject[i]->GetScript() != NULL)ListeObject[i]->GetScript()->UpdateEverySecond();
                    }
                }
            }

            for(int i = 0; i < nbobject; i++)if(ListeObject[i]->GetEnable() == true && i < nbobject)if(ListeObject[i]->GetScript() != NULL)ListeObject[i]->GetScript()->Update();

            for(int i = 0; i < nbobject; i++)if(ListeObject[i]->GetEnable() == true)if(ListeObject[i]->GetRigibody() != NULL)ListeObject[i]->GetRigibody()->AppliedForce();

            UpdateRelativePosition();

            if(gamemap != NULL)gamemap->AfficherMap();

            for(int i = 0; i< nbobject; i++)if(ListeObject[i]->GetEnable() == true)if(ListeObject[i]->GetRender() != NULL)ListeObject[i]->GetRender()->DrawObject();

            if(EScript != NULL)EScript->Update();

            if(this->fps == true)PrintV(110,2,getFps());

            setFps(this->fpswish);

            ML_display_vram();
        }

    DelAllObject();

    return 1;

}

int Engine::GetNbObject()
{
    return nbobject;
}

void Engine::SetType(int v)
{
    this->type = v;
}

Map * Engine::GetMap()
{
    return gamemap;
}

int Engine::GetType()
{
    return this->type;
}

void Engine::StopGame(){
        this->execute = false;
    }
void Engine::SetFpsWish(int v){
        this->fpswish = 180 / v;
    }
void Engine::ShowFps(){
        this->fps = true;
    }
void Engine::HideFps(){
        this->fps = false;
    }

void Engine::AddObject( Object * v)
{
    Object ** Tab = ListeObject;

    ListeObject = (Object**)malloc(sizeof(Object *) * (nbobject+1));

    for(int i = 0 ; i < nbobject  ; i++)if(Tab[i] != NULL)ListeObject[i] = Tab[i];

    this->ListeObject[ nbobject ] = v;

    nbobject ++;

    delete Tab;

    v->AffectEngine(this);
}

void Engine::DelObject( Object * v)
{
    Object ** Tab;
    int index = -1;

    Tab = this->ListeObject;

    if(Tab == NULL)
        return ;

    for(int i = 0 ; i < nbobject  ; i++)
    {
        if(Tab[i] == v ) index = i ;
    }

    if(index == -1)return;

    this->ListeObject = (Object**)malloc(sizeof(Object *) * (nbobject-1));

    if( index > 0)
         {
            for(int i = 0 ; i < index  ; i++)
            {
                this->ListeObject[ i ] = Tab[ i ];
            }
         }

     for(int i = index + 1 ; i < nbobject  ; i++)
        {
            this->ListeObject[ i - 1] = Tab[ i ];
        }

    nbobject --;

    delete v;

    delete Tab;
}

void Engine::DelAllObject()
{
    for(int i = 0 ; i < nbobject  ; i++)
    {
        delete ListeObject[i];
    }

    delete ListeObject;
    ListeObject = NULL;

}

void Engine::AffectScript(Script & VScript)
{
    this->EScript = &VScript;
    EScript->AffectEngine(this);
}

void Engine::AffectMap(Map * VMap)
{
    this->gamemap = VMap;
    VMap->AffectEngine( this );
}

bool Engine::WaitOneSecond()
{
    if(RTC_getTicks() - time > 128) // si il s'est coul une seconde complte
    {
        time = RTC_getTicks(); // et on se rappelle du nombre de ticks de la dernire seconde coule
        return true;
    }

return false;

}

void Engine::UpdateRelativePosition()
{
     for(int i = 0; i< nbobject; i++)
        {
        if(ListeObject[i]->GetEnable() == true) ListeObject[i]->GetTransforms()->SetRelativeXY( ListeObject[i]->GetTransforms()->GetX() - this->screen.min.x , ListeObject[i]->GetTransforms()->GetY() - this->screen.min.y );
        }
}

void Engine::MoveScreen(int x , int y)
{
    int w = screen.max.x - screen.min.x;
    int h = screen.max.y - screen.min.y;

    screen.min.x = x;
    screen.max.x = x + w;
    screen.min.y = y;
    screen.max.y = y + h;
}

void Engine::MiddleScreen(int x , int y , bool fixe)
{
    int w = screen.max.x - screen.min.x;
    int h = screen.max.y - screen.min.y;

    screen.min.x = x - w/2;
    screen.max.x = x + w/2;
    screen.min.y = y - h/2;
    screen.max.y = y + h/2;

    if(screen.min.x < 0 && fixe)
    {
        screen.min.x = 0;
        screen.max.x = w;
    }

    if(gamemap != NULL && fixe)
    {
        if(screen.max.x > gamemap->nbtiles_largeur_monde * gamemap->LARGEUR_TILE)
        {
            screen.max.x = gamemap->nbtiles_largeur_monde * gamemap->LARGEUR_TILE;
            screen.min.x = screen.max.x - w;
        }

        if(screen.min.y < 0)
        {
            screen.min.y = 0;
            screen.max.y = h;
        }

        if(screen.max.y > gamemap->nbtiles_hauteur_monde * gamemap->HAUTEUR_TILE)
        {
            screen.max.y = gamemap->nbtiles_hauteur_monde * gamemap->HAUTEUR_TILE;
            screen.min.y = screen.max.y - h;
        }
    }
}

AABB * Engine::GetScreen()
{
    return &screen;
}

Object ** Engine::GetListeObject()
{
    return ListeObject;
}

