#include "../C-Engine/CEngine/Object.hpp"
#include "../C-Engine/CEngine/Engine.hpp"
#include "../C-Engine/CEngine/Script.hpp"

#include "../C-Engine/CPhysics/Transforms.hpp"
#include "../C-Engine/CPhysics/Rigibodys.hpp"

#include "../C-Engine/CRendering/Render.hpp"
#include "../C-Engine/CRendering/Animation.hpp"
#include "../C-Engine/CRendering/Sprite.hpp"
#include "../C-Engine/CRendering/TilesMapping.hpp"

extern "C"
{
#include <fxlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../C-Engine/CExtraLib/input.h"
#include "../C-Engine/CExtraLib/MonochromeLib.h"
#include "../C-Engine/CExtraLib/FonctionC.h"
}

double min( double A , double B)
{
    if(A < B) return A;
    else return B;
}

Vec2 operator+ (Vec2 A , Vec2 B)
{
    Vec2 V;
    V.x = A.x + B.x;
    V.y = A.y + B.y;
    return V;
}

Vec2 operator- (Vec2 A , Vec2 B)
{
    Vec2 V;
    V.x = A.x - B.x;
    V.y = A.y - B.y;
    return V;
}

Vec2 operator* (Vec2 A , double B)
{
    Vec2 V;
    V.x = A.x * B;
    V.y = A.y * B;
    return V;
}

int Rigibody::GetHeight()
{
    return height;
}

int Rigibody::GetWidht()
{
    return widht;
}

bool Rigibody::GetStat()
{
    return usefixebody;
}

Body * Rigibody::GetBody()
{
    return &R_body;
}

Rigibody::Rigibody(Object * Pointeur)
{
    usefixebody = false;
    widht = 0;
    height = 0;

    Conteneur = Pointeur;

    R_body.transform = Pointeur->GetTransforms();

    R_body.velocity.x = 0;
    R_body.velocity.y = 0;

    R_body.acceleration.x = 0;
    R_body.acceleration.y = 0;

    R_body.mass_data.mass = 0;
    R_body.mass_data.inv_mass = 0;
}

void Rigibody::UseFixeBody(int w , int h)
{
    this->usefixebody = true;
    this->widht = w;
    this->height = h;
}

void Rigibody::UseFlexBody()
{
    this->usefixebody = false;
    this->widht = 0;
    this->height = 0;
}

bool AABBvsAABB( AABB a, AABB b )
{
// Exit with no intersection if found separated along an axis
if(a.max.x < b.min.x || a.min.x > b.max.x) return false;
if(a.max.y < b.min.y || a.min.y > b.max.y) return false;
// No separating axis found, therefor there is at least one overlapping axis
return true;
}

int Rigibody::CollisionDecor( int x , int y)
{
	if(Conteneur->GetEngine()->GetMap() != NULL)
    {
        Map * m = Conteneur->GetEngine()->GetMap();

        int sizex , sizey;

        if(this->usefixebody == false)
        {
            sizex = Conteneur->GetRender()->GetSizeX();
            sizey = Conteneur->GetRender()->GetSizeY();
        }
        else
        {
            sizex = widht;
            sizey = height;
        }


        if (    x < 0 || ( x + sizex )  > m->nbtiles_largeur_monde * m->LARGEUR_TILE ||  y + sizey > m->nbtiles_hauteur_monde * m->HAUTEUR_TILE  || y < 2)return 1;

        int xmin,xmax,ymin,ymax,i,j,indicetile, hm;

        hm = m->nbtiles_hauteur_monde * m->HAUTEUR_TILE;

        xmin = x / m->LARGEUR_TILE;
        ymin = (hm - y - sizey +1) / m->HAUTEUR_TILE;
        xmax = (x + sizex) / m->LARGEUR_TILE;
        ymax = (hm - y -1) / m->HAUTEUR_TILE;

        for( i = xmin ; i <= xmax ; i++)
        {
            for( j = ymin ; j<= ymax ; j++)
            {
                indicetile = m->map[j][i] - '0';

                if (m->plein[indicetile] == true)
                {
                    return 1;
                }
            }
        }

        return 0;
    }

    return 0;
}

int ABS( int v)
{
    if(v < 0)return -v;

    return v;
}

int SGN( int v)
{
    if(v < 0)return -1;

    return 1;
}

void Rigibody::Affine(int vx,int vy)
{
	int i;

	for( i = 0 ; i < ABS(vx) ; i++)
	{
		if ( TryMove ( SGN(vx) , 0 ) == 0 )
			break;
	}

	for( i = 0 ; i < ABS(vy) ; i++)
	{
		if ( TryMove ( 0 , SGN(vy) ) == 0 )
			break;
	}
}

int Rigibody::Move(int x,int y)
{
	if (TryMove( x, y) == 1)return 1;

    Affine(x,y);

    return 0;
}

int Rigibody::TryMove( int x ,int y)
{
	Map * m = Conteneur->GetEngine()->GetMap();

    if(m == NULL)
    {
        int tx = R_body.transform->GetX() + x;
        int ty = R_body.transform->GetY() + y;

		R_body.transform->SetXY( tx , ty);
		return 1;
    }

	if (x >= m->LARGEUR_TILE|| y>= m->HAUTEUR_TILE)
	{
		TryMove(x/2,y/2);
		TryMove(x/2, y/2);
		return 1;
	}

	int tx = R_body.transform->GetX() + x;
	int ty = R_body.transform->GetY() + y;

	if (CollisionDecor( tx , ty) == 0)
	{
		R_body.transform->SetXY( tx , ty);
		return 1;
	}
	else
    {
        if( x != 0)this->R_body.velocity.x = 0 ;
        if( y != 0)this->R_body.velocity.y = 0 ;
    }

	return 0;

}

void Rigibody::AppliedForce()
{
    if(R_body.mass_data.mass > 0)
    {
        if(Conteneur->GetEngine()->GetType() == 1)R_body.force.y = -9.81 * R_body.mass_data.mass * R_body.mass_data.mass ;

        R_body.acceleration.x = R_body.force.x * R_body.mass_data.inv_mass + R_body.acceleration.x;
        R_body.acceleration.y = R_body.force.y * R_body.mass_data.inv_mass + R_body.acceleration.y;

        R_body.velocity.x =  R_body.velocity.x + (R_body.acceleration.x * 0.1);
        R_body.velocity.y =  R_body.velocity.y + (R_body.acceleration.y * 0.1);

        Move(R_body.velocity.x * 0.1, R_body.velocity.y * 0.1);

        R_body.force.x = 0;
        R_body.force.y = 0;
        R_body.acceleration.x = 0;
        R_body.acceleration.y = 0;
    }
}

void Rigibody::SetMass(float v)
{
    this->R_body.mass_data.mass = v;
    if(v != 0)this->R_body.mass_data.inv_mass = 1/v;
    else this->R_body.mass_data.inv_mass = 0;
}


